/*

The MIT License (MIT)

Copyright (c) 2022 Rui Santana

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

=================================================================================

Based on epicycles work avaliable at http://brettcvz.github.io/epicycles/ with
the following license:

The MIT License (MIT)

Copyright (c) 2014 Brett van Zuiden

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/

const Config = {
    defaultServer: 'https://brazil.signum.network',
    server: '',
    pathColor: "#A66",
    backgroundColor: "#FFF",
    fetchingData: false,
    SmartContractId: 2523087122460988766n
}

window.onload = () => {
    Config.server = Config.defaultServer;
    requestData();
    if (window.location.search) {
        if (window.location.search.startsWith('?color=')) {
            Config.pathColor = window.location.search.split("=")[1].replace('%23','#');
        }
    }
    document.getElementById('back-color').addEventListener('input', (evt) => {
        const colorValue = evt.target.value;
        const myElements = document.querySelectorAll(".slate");
        for (let i = 0; i < myElements.length; i++) {
            myElements[i].style.backgroundColor = colorValue;
        }
    });
}

function requestData() {
    Config.fetchingData = true;
    const xhttp = new XMLHttpRequest();
    xhttp.onreadystatechange = function() {
        if(xhttp.readyState === XMLHttpRequest.DONE) {
            var status = xhttp.status;
            if (status >= 200 && status < 400) {
                // The request has been completed successfully
                Config.fetchingData = false;
                processData(xhttp.responseText);
            }
        }
    };
    xhttp.open("GET", `${Config.server}/burst?requestType=getAccountTransactions&account=${Config.SmartContractId.toString(10)}`, true);
    xhttp.send();
    setTimeout(() => {
        if (Config.fetchingData === true) {
            chooseNewServer();
        }
    }, 5500);
}

function processData(json_text) {
    let Response, Transactions;
    try {
        Response = JSON.parse(json_text);
        Transactions = Response.transactions;
    } catch (error) {
        Transactions = undefined;
    }
    if (Transactions === undefined) {
        chooseNewServer();
        return;
    }
    const Epicycles = Transactions.filter( (Tx) => {
        if (Tx.sender !== Config.SmartContractId.toString(10) ||
            Tx.attachment === undefined ||
            Tx.attachment.message.startsWith("5765622070616765") ||
            Tx.attachment.message.startsWith("0000000079a4cd83")) {
            return false;
        }
        return true;
    })
    let addresses = [];
    let epc = Epicycles[0].attachment.message.substr(0, 28);
    let sender = toggleEndianess(Epicycles[0].attachment.message.substr(32, 16));
    let senderId = BigInt("0x" + sender);
    let senderRS = idTOaccount(senderId);
    let gift = toggleEndianess(Epicycles[0].attachment.message.substr(48, 16));
    let giftId = BigInt("0x" + gift);
    let giftRS = idTOaccount(giftId);
    let epcData = decodeEPC(hexToString(epc));

    let description = `<h3>Signature of ${giftRS}<span name="${giftId}"></span></h3>`;
    addresses.push(giftId);
    if (senderId !== giftId) {
        description += `<p>A gift from ${senderRS}<span name="${senderId}"></span></p>`;
        addresses.push(senderId)
    }
    document.getElementById('last').innerHTML = `${description}
    <div class="epicycle-display" id="ep0" data-points="${epcData}"
            data-name="circle" data-thick-lines="false" data-simulator-controls="true" data-show-rods="false" data-show-path="true" data-grid-controls="true" data-show-grid="false"  data-show-disks="false" data-show-points="false"
            data-width="700" data-height="700" data-scale="40" data-speed="1" data-color="${Config.pathColor}"></div>`
    setTimeout(showEpicycle, 1000, 'ep0');
    
    if (Epicycles.length === 1) {
        document.getElementById('previous').innerHTML = 'No more signatures.';
        return;
    }
    let previousEpicycles = '';
    for (let i = 1; i < Epicycles.length; i++) {
        epc = Epicycles[i].attachment.message.substr(0, 28);
        sender = toggleEndianess(Epicycles[i].attachment.message.substr(32, 16));
        senderId = BigInt("0x"+sender);
        senderRS = idTOaccount(senderId);
        gift = toggleEndianess(Epicycles[i].attachment.message.substr(48, 16));
        giftId = BigInt("0x"+gift);
        giftRS = idTOaccount(giftId);
        epcData = decodeEPC(hexToString(epc));
    
        description = `<h3 onclick='javascript:showEpicycle("ep${i}");'>Signature of ${giftRS}<span name="${giftId}"></span></h3>`;
        addresses.push(giftId);
        if (senderId !== giftId) {
            description += `<p>A gift from ${senderRS}<span name="${senderId}"></span></p>`;
            addresses.push(senderId);
        }
        previousEpicycles += `${description}
            <div class="epicycle-display" id="ep${i}" data-points="${epcData}"
            data-name="circle" data-thick-lines="false" data-simulator-controls="true" data-show-rods="false" data-show-path="true" data-grid-controls="true" data-show-grid="false"  data-show-disks="false" data-show-points="false"
            data-width="700" data-height="700" data-scale="40" data-speed="1" data-color="${Config.pathColor}"></div>`
        
    }

    document.getElementById('previous').innerHTML = previousEpicycles;

    /* request owners details */
    let owners = addresses.reduce( (arr, item) => { 
        if (arr.indexOf(item)==-1) {
            arr.push(item);
        }
        return arr;
    }, []);
    owners.forEach( function (own){
        const xhttp = new XMLHttpRequest();
        xhttp.onreadystatechange = function() {
            if (this.readyState == 4 && this.status == 200) {
                // Typical action to be performed when the document is ready:
                processOwner(xhttp.responseText);
            }
        };
        xhttp.open("GET", `${Config.server}/burst?requestType=getAccount&account=${own}`, true);
        xhttp.send();
    })
    
}

function processOwner(json_text) {
    const ownerInfo=JSON.parse(json_text)
    if (ownerInfo.name === undefined || ownerInfo.name === "") {
        return
    }
    const spans = document.getElementsByName(ownerInfo.account)
    spans.forEach( dom => {
        dom.innerHTML = ` - ${ownerInfo.name.slice(0,30)}`;
    })
}

function showEpicycle(epId) {
    let display = new GridDisplay();
    display.loadAndRenderInto(document.getElementById(epId));
}

function hexToString(hex) {
    let ret = "";
    for (let c = 0; c < hex.length; c += 2) {
        ret += String.fromCharCode(parseInt(hex.substr(c, 2), 16));
    }
    return ret;
}

function decodeEPC(baseABC) {
    let chars = baseABC.split('');
    let digits = chars.map((ch) => ((ch.charCodeAt() - 65)* 4).toString(10).padStart(2, "0") + "0");
    return digits.join('');
}

function toggleEndianess(inp) {
    if (/^[a-fA-F0-9]{16}/.exec(inp) === null) {
        return '#error#';
    }
    let inpArr = inp.split('');
    let retNo = [];
    for (let i = 0; i < 8; i++) {
        retNo.push(inpArr[14 - (i * 2)] + inpArr[15 - (i * 2)]);
    }
    return retNo.join('');
}

//Input id in unsigned long (BigInt)
//Output string with account address (Reed-Salomon encoded)
function idTOaccount(id) {

    let gexp = [1, 2, 4, 8, 16, 5, 10, 20, 13, 26, 17, 7, 14, 28, 29, 31, 27, 19, 3, 6, 12, 24, 21, 15, 30, 25, 23, 11, 22, 9, 18, 1]
    let glog = [0, 0, 1, 18, 2, 5, 19, 11, 3, 29, 6, 27, 20, 8, 12, 23, 4, 10, 30, 17, 7, 22, 28, 26, 21, 25, 9, 16, 13, 14, 24, 15]
    let cwmap = [3, 2, 1, 0, 7, 6, 5, 4, 13, 14, 15, 16, 12, 8, 9, 10, 11]
    let alphabet = "23456789ABCDEFGHJKLMNPQRSTUVWXYZ".split("")
    let base32alpha="0123456789abcdefghijklmnopqrstuv"
    let base32Length = 13
    let account = "S-"
    let i;
    
    function ginv(a) {
        return gexp[31 - glog[a]];
    }
    
    function gmult(a, b) {
        if (a == 0 || b == 0) {
            return 0;
        }
        return gexp[ (glog[a] + glog[b]) % 31 ]
    }
    
    if (id == 0){
        return ""
    }
    
    base32=id.toString(32).padStart(13,"0").split("")
    var codeword=[]
    for (i=0; i<base32Length; i++){
        codeword.push( base32alpha.indexOf(base32[12-i]) );
    }
    
    var p = [0, 0, 0, 0]
    for (i=base32Length-1; i>=0; i--) {
        let fb = codeword[i] ^ p[3]
        
        p[3] = p[2] ^ gmult(30, fb)
        p[2] = p[1] ^ gmult(6, fb)
        p[1] = p[0] ^ gmult(9, fb)
        p[0] = gmult(17, fb)
    }
    codeword.push(p[0], p[1], p[2], p[3])
    
    for (i=0; i<17; i++) {
        account+=alphabet[codeword[cwmap[i]]]
        if ((i & 3) == 3 && i < 13) {
            account+="-"
        }
    }
    
    return account
}

function chooseNewServer() {
    Config.server = window.prompt(`Server ${Config.server} is not responding. Enter another one. Some possibilities:\n\n` +
        'https://brazil.signum.network\nhttps://uk.signum.network\nhttps://europe.signum.network\nhttps://australia.signum.network\nhttp://localhost:8125',);
        // Config.server = Config.server2;
    if (Config.server === null || Config.server === '') {
        Config.server = Config.defaultServer;
        return;
    }
    requestData();
}
