function Simulator(grid, gears, config) {
    this.grid = grid;
    this.gears = gears;
    this.config = config;
    // this.speed = 0.3; //speed || 0.3;
    this.reset();

    this.renderSteps = [];
    // this.reset();
    // this.renderLoop();
};

Simulator.prototype.addRenderStep = function(cb) {
    this.renderSteps.push(cb);
};

Simulator.prototype.renderLoop = function(){
    //Not re-entrant, should only be called once
    var TWO_PI = 2*Math.PI;

    var last = null;
    var sim = this;
    var step = function(now) {
        if (sim.playing) {
            if (last !== null) {
                sim.t = (sim.t + ((now - last) / 1000 * sim.config.speed));
                if (sim.t > TWO_PI) {
                    sim.playing = false;
                }
            }
            last = now;
            sim.render();
            requestAnimationFrame(step);
        } else {
            last = null;
        }
    }

    requestAnimationFrame(step);
}

Simulator.prototype.play = function(){
    this.playing = true;
    this.renderLoop();
};

Simulator.prototype.pause = function(){
    this.playing = false;
};

Simulator.prototype.step = function(dt){
    var TWO_PI = Math.PI * 2;
    this.t = (this.t + dt) % TWO_PI;
    if (this.t < 0) { this.t += TWO_PI;}
    this.render();
};

Simulator.prototype.setGears = function(gears){
    this.pause();
    this.gears = gears;
    this.reset();
    this.render();
};

Simulator.prototype.reset = function(){
    this.t = 0;
    this.playing = false;
    this.path = [];
    this.grid.clear();
};

Simulator.prototype.render = function() {
    this.grid.clear();
    this.renderSteps.forEach(function(cb){cb();});

    var offsetX = 0, offsetY = 0;
    var t = t || 0;
    var colors = ["#33CC33", "#CC3333", "#3333CC",
            "#0066CC", "#CC0066", "#66CC00",
            "#00CC66", "#CC6600", "#6600CC"];
    for (var i = 0; i < this.gears.length; i++) {
        var gear = this.gears[i];
        var endX = offsetX + gear[1] * Math.cos(gear[0] * this.t + gear[2]);
        var endY = offsetY + gear[1] * Math.sin(gear[0] * this.t + gear[2]);
        if (this.config.showDisks) {
            this.grid.drawCircle(offsetX, offsetY, gear[1], colors[i % colors.length]);
        }
        if (this.config.showRods) {
            this.grid.drawStick(offsetX, offsetY, endX, endY, colors[i % colors.length], true);
        }
        offsetX = endX;
        offsetY = endY;
    }

    //Save the final point for drawing later
    this.path.push([offsetX, offsetY]);

    if (this.config.showPath) {
        this.drawPath();
    }
};

Simulator.prototype.drawPath = function(){
    var grid = this.grid;
    this.path.reduce((prev, curr) => {
        grid.drawStick(prev[0], prev[1], curr[0], curr[1], this.config.color, false);
        return curr;
    })
};


